@@ -11,6 +11,23 @@ silhouette.partition <- function(x, ...) {
     r
 }
 
+silhouette.clara <- function(x, full = FALSE, ...)
+{
+    if(!full)
+	return(NextMethod()) ##-> silh*.partition()
+
+    ## else : full = TRUE
+    if(is.null(x$data))
+	stop("full silhouette is only available for results of",
+	     " 'clara(*, keep.data = TRUE)'")
+    ## Compute "full" silhouette -- from clustering + full distances:
+    r <- silhouette(x$clustering,
+		    daisy(x$data, metric = attr(x, "Metric")))
+    attr(r, "call") <-
+	substitute(silhouette(CL, full = TRUE), list(CL = x$call))
+    r
+}
+
 silhouette.default <- function(x, dist, dmatrix, ...) {
     cll <- match.call()
     if(!is.null(cl <- x$clustering)) x <- cl
@@ -36,15 +53,20 @@ silhouette.default <- function(x, dist, dmatrix, ...) {
     for(j in 1:k) { # j-th cluster:
         Nj <- sum(iC <- x == clid[j])
         wds[iC, "cluster"] <- clid[j]
-        a.i <- if(Nj > 1) colSums(dmatrix[iC, iC])/(Nj - 1) else 0
-                                        # length(a.i)= Nj
         ## minimal distances to points in all other clusters:
         diC <- rbind(apply(dmatrix[!iC, iC, drop = FALSE], 2,
                            function(r) tapply(r, x[!iC], mean)))# (k-1) x Nj
-        minC <- max.col(-t(diC))
+        ## max.col() breaks ties at random;  rather do not want random
+        ##   behavior of silhouette, (but rather "pam" compatible one):
+        minC <- apply(diC, 2, which.min)
+        ## FIXME minC <- max.col(-t(diC))
+        ## FIXME : extend max.col(*, ties.method = "min") {or similar} !
         wds[iC,"neighbor"] <- clid[-j][minC]
-        b.i <- diC[cbind(minC, seq(along = minC))]
-        s.i <- ifelse(a.i != b.i, (b.i - a.i) / pmax(b.i, a.i), 0)
+        s.i <- if(Nj > 1) {
+            a.i <- colSums(dmatrix[iC, iC])/(Nj - 1) # length(a.i)= Nj
+            b.i <- diC[cbind(minC, seq(along = minC))]
+            ifelse(a.i != b.i, (b.i - a.i) / pmax(b.i, a.i), 0)
+        } else 0
         wds[iC,"sil_width"] <- s.i
     }
     attr(wds, "Ordered") <- FALSE
@@ -116,8 +138,9 @@ plot.silhouette <-
     s <- rev(x[, "sil_width"])
     space <- c(0, rev(diff(cli <- x[, "cluster"])))
     space[space != 0] <- 0.5 # gap between clusters
-    names <- if(n < nmax.lab)
-	substring(rev(rownames(x)), 1, max.strlen)
+    axisnames <- (n < nmax.lab)
+    if(axisnames)
+        names <- substring(rev(rownames(x)), 1, max.strlen)
     if(is.null(main)) {
 	main <- "Silhouette plot"
 	if(!is.null(cll <- attr(x,"call"))) { # drop initial "silhouette":
@@ -141,7 +164,8 @@ plot.silhouette <-
     y <- barplot(s, space = space, names = names, xlab = xlab,
 		 xlim = c(min(0, min(s)), 1),
 		 horiz = TRUE, las = 1, mgp = c(2.5, 1, 0),
-		 col = col, border = border, cex.names = cex.names, ...)
+		 col = col, border = border, cex.names = cex.names,
+                 axisnames = axisnames, ...)
     title(main = main, sub = sub, adj = 0)
     if(do.n.k) {
 	mtext(paste("n =", n),	adj = 0)
@@ -153,8 +177,9 @@ plot.silhouette <-
 	y <- rev(y)
 	for(j in 1:k) {
 	    yj <- mean(y[cli == j])
-	    text(1, yj, paste(j,":  ", nj[j]," | ",
-			      format(smry$clus.avg.widths[j], digits = 2)),
+	    text(1, yj,
+                 paste(j,":  ", nj[j]," | ",
+                       format(smry$clus.avg.widths[j], digits = 1, nsmall = 2)),
 		 xpd = NA, adj = 0.8)
 	}
     }
